<?php
/*
Plugin Name: Side Matter
Plugin URI: http://wordpress.org/extend/plugins/side-matter/
Description: Turns your footnotes into sidenotes, magically positioning each note in the sidebar next to its corresponding reference in the text.
Version: 0.8
Author: Christopher Setzer
Author URI: http://christophersetzer.com
License: GPLv2
*/

/*
Copyright (C) 2013  Christopher Setzer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

$side_matter = new Side_Matter;

class Side_Matter {

	public $options = array(); // Side Matter settings as pulled from database field side_matter_options (array)
	public $defaults = array(); // Option defaults (array)
	public $version; // Current version of Side Matter

	public $notes = array(); // Container for Side Matter notes (array)
	public $note_count; // Count of notes; used as key in $notes (integer)

	public function __construct() { // Construct class and set defaults

		add_action( 'admin_init', array( &$this, 'settings' ) ); // Register settings section
		add_action( 'widgets_init', array( &$this, 'widget' ) ); // Register widget
		add_shortcode( 'ref', array( &$this, 'shortcode' ) ); // Register shortcode
		add_action( 'wp_enqueue_scripts', array( &$this, 'enqueue' ) ); // Enqueue script and stylesheet; embed prefs in side-matter.js
		add_action( 'side_matter_list_notes', array( &$this, 'list_notes' ) ); // Get HTML list of notes

		$this->defaults = array( // Set option defaults
			'is_responsive' => 1, // Responsive sidenotes (boolean; default: 1)
			'use_effects' => 1, // Enable fade effects (boolean; default: 1)
			'note_adjust' => 0, // Arbitrary note offset, in px (integer; default: 0; accepts positive and negative numbers)
			'html_class' => sanitize_html_class( 'side-matter' ), // Class to use for Side Matter tags (string; default: 'side-matter')
			'pages_active' => array( // Pages on which to display Side Matter notes (array)
				'front' => 0, // Default: 0
				'home' => 0, // Default: 0
				'post' => 1, // Default: 1
				'page' => 1 // Default: 1
				)
			);

		$this->options = wp_parse_args( get_option( 'side_matter_options' ), $this->defaults ); // Get options from database; apply defaults in place of any that are not set
		$this->version = '0.8'; // Set current version of Side Matter
		$this->note_count = 0; // Set number from which to begin note count (integer; default: 0)

	}

	public function settings() { // Register Side Matter section and fields
		$page = 'reading';
		$section = 'side_matter_section';
		$fields = array(
			'pages_active' => array(
				'id' => 'pages_active',
				'title' => 'Display Side Matter notes on',
				'callback' => array( &$this, 'pages_active_field' ),
				'args' => array(
					'pages' => array(
						'front' => 'Front page',
						'home' => 'Posts page',
						'post' => 'Single posts',
						'page' => 'Single pages'
						)
					)
				),
			'is_responsive' => array(
				'id' => 'is_responsive',
				'title' => 'Responsive Sidenotes',
				'callback' => array( &$this, 'is_responsive_field' ),
				'args' => array(
					'label_for' => 'is_responsive',
					'text_append' => 'Enable responsive positioning on page resize and zoom'
					)
				),
			'use_effects' => array(
				'id' => 'use_effects',
				'title' => 'Transition Effects',
				'callback' => array( &$this, 'use_effects_field' ),
				'args' => array(
					'label_for' => 'use_effects',
					'text_append' => 'Enable fade effects on page load, resize, and zoom'
					)
				),
			'note_adjust' => array(
				'id' => 'note_adjust',
				'title' => 'Adjust vertical offset of sidenotes by',
				'callback' => array( &$this, 'note_adjust_field' ),
				'args' => array(
					'label_for' => 'note_adjust',
					'text_append' => 'px'
					)
				)
		);
		add_settings_section( 'side_matter_section', 'Side Matter', array( &$this, 'section' ), $page );
		foreach ( $fields as $option => $parameter ) {
			add_settings_field( $fields[$option]['id'], $fields[$option]['title'], $fields[$option]['callback'], $page, $section, $fields[$option]['args'] );
		}
		register_setting( $page, 'side_matter_options', array( &$this, 'validate' ) );
	}
	
	public function section() { // Text to place at beginning of settings section
		echo 'Use the following options to control the appearance and functionality of notes generated by the Side Matter plugin.';
	}

	public function pages_active_field( $args ) { // Field for pages_active option
		foreach ( $args['pages'] as $page => $page_name ) {
			echo "<label><input type='checkbox' name='side_matter_options[pages_active][{$page}]' value='1' " . checked( 1, $this->options['pages_active'][$page], false ) . " /> {$page_name}</label><br />";
		}
		echo '<p class="description">These conditions may override one another in some instances, as when a static page is set as your front page.</p>';
	}

	public function is_responsive_field( $args ) { // Field for is_responsive option
		echo "<p><label><input id='is_responsive' type='checkbox' name='side_matter_options[is_responsive]' value='1' " . checked( 1, $this->options['is_responsive'], false ) . ' /> ' . $args['text_append'] . '</label></p>';
	}

	public function use_effects_field( $args ) { // Field for use_effects option
		echo "<p><label><input id='use_effects' type='checkbox' name='side_matter_options[use_effects]' value='1' " . checked( 1, $this->options['use_effects'], false ) . ' /> ' . $args['text_append'] . '</label></p>';
	}

	public function note_adjust_field( $args ) { // Field for note_adjust option
		echo '<label><input id="note_adjust" name="side_matter_options[note_adjust]" type="number" step="1" max="1000" min="-1000" value="' . $this->options['note_adjust'] . '" class="small-text" /> ' . $args['text_append'] . '</label><p class="description">A positive value will push notes up; a negative value will push notes down.</p>';
	}

	public function validate( $input ) { // Validate and sanitize field input from settings section
		$effects = array( 'is_responsive', 'use_effects' );
		$pages = $this->defaults['pages_active'];
		foreach ( $pages as $page => $setting ) {
			$this->options['pages_active'][$page] = ( ! isset( $input['pages_active'][$page] ) ) ? 0 : $input['pages_active'][$page];
		}
		foreach ( $effects as $effect ) {
			$this->options[$effect] = ( ! isset( $input[$effect] ) ) ? 0 : $input[$effect];
		}
		$this->options['note_adjust'] = ( ! isset( $input['note_adjust'] ) ) ? $this->defaults['note_adjust'] : intval( $input['note_adjust'] );
		return $this->options;
	}

	public function widget() { // Register widget
		register_widget( "Side_Matter_Widget" );
	}

	public function shortcode( $atts, $content = null ) { // Capture shortcoded text and return reference numeral
		$class = $this->options['html_class'];
		if ( $this->check_page() ) {
			$this->add_note( $content );
			return "<a id='ref-{$this->note_count}' class='{$class} {$class}-ref {$class}-ref-{$this->note_count}' href='#note-{$this->note_count}'><sup class='{$class} {$class}-sup {$class}-sup-{$this->note_count}'>{$this->note_count}</sup></a>";
		} else {
			return '';
		}
	}

	public function enqueue() { // Enqueue script and stylesheet; embed settings in side-matter.js
		if ( $this->check_page() ) {
			wp_enqueue_script( 'side-matter-js', plugins_url( 'side-matter.js', __FILE__ ), array( 'jquery' ), $this->version, true );
			wp_localize_script( 'side-matter-js', 'side_matter', array( // Embed settings in side-matter.js
				'is_responsive' => __( $this->options['is_responsive'] ),
				'use_effects' => __( $this->options['use_effects'] ),
				'note_adjust' => __( $this->options['note_adjust'] ),
				'html_class' => __( $this->options['html_class'] ) )
				);
			wp_enqueue_style( 'side-matter', plugins_url( 'side-matter.css', __FILE__ ), null, $this->version );
		}
	}

	public function check_page() { // If Side Matter is set to display notes on current page, return true

		$pages = $this->options['pages_active'];

		$front = ( is_front_page() && $pages['front'] ) ? 1 : 0;
		$home = ( is_home() && $pages['home'] ) ? 1 : 0;
		$post = ( is_single() && $pages['post'] ) ? 1 : 0;
		$page = ( is_page() && $pages['page'] ) ? 1 : 0;
		$pages_table = array( $front, $home, $post, $page );

		if ( in_array( 1, $pages_table ) ) return true;

	}

	public function add_note( $shortcode_content ) { // Accept shortcoded text and add it to array $notes
		$this->note_count++;
		$this->notes[$this->note_count] = $shortcode_content; // Add note to end of array $notes
	}

	public function list_notes() { // Generate HTML-formatted list of notes
		$class = $this->options['html_class'];
		if ( $this->check_page() && isset( $this->notes ) ) {
			$notes_list = "<ol class='{$class} {$class}-list {$class}-ol '>";
			foreach ( $this->notes as $id => $text ) {
				$notes_list .= "<li id='note-{$id}' class='{$class} {$class}-note {$class}-note-{$id} {$class}-li {$class}-li-{$id}'><div class='{$class} {$class}-text {$class}-text-{$id}'>" . wpautop( $text ) . "</div></li>";
			}
			$notes_list .= '</ol>';
			echo $notes_list;
		}
	}

}

class Side_Matter_Widget extends WP_Widget {

	public function __construct() {
		parent::__construct( 'side-matter', __( 'Side Matter' ), array( 'description' => __( 'Display Side Matter notes in an ordered list' ) ) );
	}

	public function widget( $args, $instance ) { // Output widget content
		global $side_matter;
		$class = $side_matter->options['html_class'];
		echo "<div id='{$this->id}' class='widget {$class} {$class}-widget'>";
		do_action( 'side_matter_list_notes' ); // Call list_notes() in widget-enclosed format
		echo '</div>';
	}

	public function form( $instance ) { // Widget options form
		$url = admin_url( 'options-reading.php' );
		echo "Modify Side Matter's default settings using the options menu on your <a href='{$url}'>Reading Settings</a> screen.";
	}

}
